/***********************************************************************
 *	This file contains functions that convert RGB colour images
 *	into other formats such as HSI, HSV, YIQ, etc.
 ***********************************************************************/
#include "incl.h"

#define MIN(A, B)	(((A) < (B)) ? (A) : (B))

int CIPL_rgbtohsi(THEADER *, THEADER *);
int CIPL_hsitorgb(THEADER *, THEADER *);

/***********************************************************************
 *	NAME: CIPL_rgbtohsi
 *	AUTHOR: Chakravarthy Bhagvati (23 November 2003)
 *	PROTOTYPE: int CIPL_rgbtohsi(THEADER *it, THEADER *ot)
 *	DESCRIPTION: This function converts an RGB image into HSI
 *		format. The parameter ot should be previously
 *		initialized to PPM format. Hue value is stored in
 *		ot->jarr field, Saturation in ot->karr field and 
 *		Value in ot->iarr field. Storing Value in ot->iarr
 *		allows all IPTK monochrome operations to be directly
 *		applied on ot.
 *
 *		Hue values are stored as 10 * hue angle in degrees so
 *		that we get accuracy to 1st decimal place. The range
 *		of hue values is 0 - 3600.
 *
 *		Saturation values are stored as 1000 * saturation so
 *		that we get accuracy to 3rd decimal place. The range
 *		of saturation values is 0 - 1000.
 *
 *		Values are stored in the range 0 - 255 without any
 *		scaling for compatibility with IPTK monochrome functions.
 *
 *	RETURN VALUE: 0 if successful, non-zero upon failure
 ***********************************************************************/
int CIPL_rgbtohsi(THEADER *it, THEADER *ot)
{
     int i, j, max;
     double R, G, B, H, S, I;
     ot->img.TYPE = HSV;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }

	  for (i=0; i<max; i++) {
	       for (j=0; j<it->img.COLS; j++) {
		    R = it->iarr[i][j];
		    G = it->jarr[i][j];
		    B = it->karr[i][j];
		    /* iarr stores v component scaled to range 0 - 255 */
		    I = (R + G + B) / 3.0;
		    ot->iarr[i][j] = (int)(I + 0.5);
		    /* karr stores s component scaled to range 0 - 1000 */
		    if ((int)(R + G + B) == 0)
			 S = 0.0;
		    else
			 S = 1.0 - (3.0 * MIN(R, MIN(G, B)) / (R + G + B));
		    ot->karr[i][j] = (int)(1000 * S);
		    /* jarr stores h component scaled to range 0 - 3600 */
		    if (ot->karr[i][j] > 0) {
			 H = 0.5 * ((R - G) + (R - B)) / 
			      (sqrt((R - G) * (R - G) + (R - B) * (G - B)));
			 if ((int)(B) > (int)(G))
			      H = (2.0 * M_PI - acos(H)) * 180.0 / M_PI;
			 else
			      H = acos(H) * 180.0 / M_PI;
		    } else
			 H = 0;
		    ot->jarr[i][j] = (int)(10 * H);

		    /* Just for debugging 
		    if ((i == 20) && (j == 20)) {
			 printf(" %d :: R, G, B: %d, %d, %d\t\t", i,(int)R,
				(int)G, (int)B);
			 printf("H, S, I: %d, %d, %d\n", ot->jarr[i][j],
				ot->karr[i][j], ot->iarr[i][j]);
		    }*/
	       }
	  }

	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }
	  
	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

/***********************************************************************
 *	NAME: CIPL_hsitorgb
 *	AUTHOR: Chakravarthy Bhagvati (23 November 2003)
 *	PROTOTYPE: int CIPL_hsitorgb(THEADER *it, THEADER *ot)
 *	DESCRIPTION: This function converts an HSI image into RGB
 *		format. The parameter ot should be previously
 *		initialized to PPM format. Green value is stored in
 *		ot->jarr field, Blue in ot->karr field and 
 *		Red in ot->iarr field. 
 *
 *	RETURN VALUE: 0 if successful, non-zero upon failure
 ***********************************************************************/
int CIPL_hsitorgb(THEADER *it, THEADER *ot)
{
     int i, j, max;
     double R, G, B, H, S, I;
     ot->img.TYPE = PPM;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }

	  for (i=0; i<max; i++) {
	       for (j=0; j<it->img.COLS; j++) {
		    I = it->iarr[i][j];
		    H = it->jarr[i][j];
		    S = it->karr[i][j];
		    I = I * 3;		       /* Scale back to original */
		    H = H / 10;		       /* Scale back to 0 - 360 */
		    S = S / 1000;	       /* Scale back to 0 - 1 */

		    if (H == 0.0)
			 H = 360;

		    /* 0 < H <= 120 degrees case */
		    if ((H > 0) && (H <= 120)) {
			 B = (1.0 - S) / 3.0;
			 R = (1.0 + (S * cos(H * M_PI / 180.0)) / 
			      cos(M_PI * (60.0 - H) / 180.0)) / 3.0;
			 G = 1.0 - (R + B);

		    /* 120 < H <= 240 degrees case */
		    } else if ((H > 120) && (H <= 240)) {
			 H = H - 120;
			 R = (1.0 - S) / 3.0;
			 G = (1.0 + (S * cos(H * M_PI / 180.0)) / 
			      cos(M_PI * (60.0 - H) / 180.0)) / 3.0;
			 B = 1.0 - (R + G);
		    /* 240 < H <= 360 degrees case */
		    } else {
			 H = H - 240;
			 G = (1.0 - S) / 3.0;
			 B = (1.0 + (S * cos(H * M_PI / 180.0)) / 
			      cos(M_PI * (60.0 - H) / 180.0)) / 3.0;
			 R = 1.0 - (B + G);
		    }

		    ot->iarr[i][j] = (int)(R * I);
		    ot->jarr[i][j] = (int)(G * I);
		    ot->karr[i][j] = (int)(B * I);

		    /* These checks are needed because of limited 
		       precision for hue and saturation values and
		       I observed that values can be as high as
		       261 on some occasions */
		    if (ot->iarr[i][j] > 255)
			 ot->iarr[i][j] = 255;
		    if (ot->jarr[i][j] > 255)
			 ot->jarr[i][j] = 255;
		    if (ot->karr[i][j] > 255)
			 ot->karr[i][j] = 255;

		    /* Just for debugging 
		    if ((i == 20) && (j == 20)) {
			 printf("H, S, V: %d, %d, %d\t\t", it->jarr[i][j],
				it->karr[i][j], it->iarr[i][j]);
			 printf("R, G, B: %d, %d, %d\n", ot->iarr[i][j],
				ot->jarr[i][j], ot->karr[i][j]);
		    }*/
	       }
	  }
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }
	  
	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}
