/***********************************************************************
 *	This file contains functions that convert RGB colour images
 *	into the XYZ format .
 ***********************************************************************/
#include "incl.h"

int CIPL_rgbtoxyz(THEADER *, THEADER *);
int CIPL_xyztorgb(THEADER *, THEADER *);

/***********************************************************************
 *	NAME: CIPL_rgbtoxyz
 *	AUTHOR: Chakravarthy Bhagvati (23 November 2003)
 *	PROTOTYPE: int CIPL_rgbtoxyz(THEADER *it, THEADER *ot)
 *	DESCRIPTION: This function converts an RGB image into XYZ
 *		format. The parameter ot should be previously
 *		initialized to PPM format. Y value is stored in
 *		ot->jarr field, Z in ot->karr field and X Value
 *		in ot->iarr field.
 *		
 *		The range of X, Y, Z values is 0 - 255.
 *
 *
 *		Values are stored in the range 0 - 255 without any
 *		scaling for compatibility with IPTK monochrome functions.
 *
 *	RETURN VALUE: 0 if successful, non-zero upon failure
 ***********************************************************************/
int CIPL_rgbtoxyz(THEADER *it, THEADER *ot)
{
     int i, j, max;
     double R, G, B, X, Y, Z;
     ot->img.TYPE = XYZ;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }

	  for (i=0; i<max; i++) {
	       for (j=0; j<it->img.COLS; j++) {
		    R = it->iarr[i][j];
		    G = it->jarr[i][j];
		    B = it->karr[i][j];
		    /* iarr stores Y component scaled to range 0 - 255 */
		    Y = 0.212671 * R + 0.715160 * G + 0.072169 * B;
		    ot->iarr[i][j] = (int)(Y+0.5);
		    /* jarr stores X component scaled to range 0 - 255 */
		    X = 0.412453 * R + 0.357580 * G  + 0.180423 * B;
		     ot->jarr[i][j] = (int)(X+0.5);
		    /* karr stores Z component scaled to range 0 - 255 */
		     Z = 0.019334 * R  +0.119193 * G + 0.950227 * B;
		    ot->karr[i][j] = (int) (Z+0.5);
			/*Debugging 
		    if ((i == 46) && (j == 206)) {
			 printf("R, G, B: %d, %d, %d\t\t", (int)R,
				(int)G, (int)B);
			 printf("X, Y, Z : %d, %d, %d\n", ot->jarr[i][j],
				ot->iarr[i][j], ot->karr[i][j]);
		    }*/
	       }
	  }

	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }
	  
	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

/***********************************************************************
 *	NAME: CIPL_xyztorgb
 *	AUTHOR: Chakravarthy Bhagvati (23 November 2003)
 *	PROTOTYPE: int CIPL_xyztorgb(THEADER *it, THEADER *ot)
 *	DESCRIPTION: This function converts an XYZ image into RGB
 *		format. The parameter ot should be previously
 *		initialized to PPM format. Green value is stored in
 *		ot->jarr field, Blue in ot->karr field and 
 *		Red in ot->iarr field. 
 *
 *	RETURN VALUE: 0 if successful, non-zero upon failure
 ***********************************************************************/
int CIPL_xyztorgb(THEADER *it, THEADER *ot)
{
     int i, j, max;
     double R, G, B, X, Y, Z;
     ot->img.TYPE = PPM;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }

	  for (i=0; i<max; i++) {
	       for (j=0; j<it->img.COLS; j++) {
		    Y = it->iarr[i][j];
		    X = it->jarr[i][j];
		    Z = it->karr[i][j];
		    R = 3.240479 * X - 1.537150 * Y - 0.498535 * Z;
		    ot->iarr[i][j] = (int)(R+0.5);
		    G = -0.969256 * X  + 1.875992 * Y  + 0.041556 * Z;
		    ot->jarr[i][j] = (int)(G+0.5);
		    B = 0.055648 * X  - 0.204043 * Y + 1.057311 * Z;
		    ot->karr[i][j] = (int)(B+0.5);

		    /* These checks are needed because of limited 
		       precision for X, Y and Z values 
		       observed that values can be as high as
		       261 on some occasions */
		    if (ot->iarr[i][j] > 255)
			 ot->iarr[i][j] = 255;
		    if (ot->jarr[i][j] > 255)
			 ot->jarr[i][j] = 255;
		    if (ot->karr[i][j] > 255)
			 ot->karr[i][j] = 255;
			 
		    if (ot->iarr[i][j] < 0)
			 ot->iarr[i][j] = 0;
		    if (ot->jarr[i][j] < 0)
			 ot->jarr[i][j] = 0;
		    if (ot->karr[i][j] < 0)
			 ot->karr[i][j] = 0;
	 
		    /* Just for debugging 
		    if ((i == 46) && (j == 206)) {
			 printf("X, Y, Z: %d, %d, %d\t\t", it->jarr[i][j],
				it->iarr[i][j], it->karr[i][j]);
			 printf("R, G, B: %d, %d, %d\n", ot->iarr[i][j],
				ot->jarr[i][j], ot->karr[i][j]);
		    }*/
	       }
	  }
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }
	  
	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}
