/***********************************************************************
 *	This file contains functions that convert RGB colour images
 *	into the YIQ (Luminance & Chrominances)formats.
 ***********************************************************************/
#include "incl.h"

int CIPL_rgbtoyiq(THEADER *, THEADER *);
int CIPL_yiqtorgb(THEADER *, THEADER *);

/***********************************************************************
 *	NAME: CIPL_rgbtoyiq
 *	AUTHOR: Chakravarthy Bhagvati (23 November 2003)
 *	PROTOTYPE: int CIPL_rgbtoyiq(THEADER *it, THEADER *ot)
 *	DESCRIPTION: This function converts an RGB image into HSI
 *		format. The parameter ot should be previously
 *		initialized to PPM format.  I value is stored in
 *		ot->jarr field, Q in ot->karr field and Y(Luminance)
 *		Value in ot->iarr field. Storing Y Value in ot->iarr
 *		allows all IPTK monochrome operations to be directly
 *		applied on ot.
 *
 *	 	The range of Y, I and Q values is 0 - 255.
 *		
 *		Values are stored in the range 0 - 255 without any
 *		scaling for compatibility with IPTK monochrome functions.
 *
 *	RETURN VALUE: 0 if successful, non-zero upon failure
 ***********************************************************************/
int CIPL_rgbtoyiq(THEADER *it, THEADER *ot)
{
     int i, j, max;
     double R, G, B, Y, I, Q;
     ot->img.TYPE = YIQ;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }

	  for (i=0; i<max; i++) {
	       for (j=0; j<it->img.COLS; j++) {
		    R = it->iarr[i][j];
		    G = it->jarr[i][j];
		    B = it->karr[i][j];
		    /* iarr stores Y component scaled to range 0 - 255 */
		    Y = 0.299 * R + 0.587 * G + 0.114 * B;
		    ot->iarr[i][j] = (int)(Y+0.5);
		    /* jarr stores I component scaled to range 0 - 255 */
		    I = 0.596 * R -0.275 * G  -0.321 * B;
		     ot->jarr[i][j] = (int)(I+0.5);
		    /* karr stores Q component scaled to range 0 - 255 */
		     Q = 0.212 * R  -0.523 * G + 0.311 * B;
		    ot->karr[i][j] = (int) (Q+0.5);
			/*Debugging 
		    if ((i == 15) && (j == 15)) {
			 printf("R, G, B: %d, %d, %d\t\t", (int)R,
				(int)G, (int)B);
			 printf("y, I, Q: %d, %d, %d\n", ot->iarr[i][j],
				ot->jarr[i][j], ot->karr[i][j]);
		    }*/
	       }
	  }

	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }
	  
	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}

/***********************************************************************
 *	NAME: CIPL_yiqtorgb
 *	AUTHOR: Chakravarthy Bhagvati (23 November 2003)
 *	PROTOTYPE: int CIPL_yiqtorgb(THEADER *it, THEADER *ot)
 *	DESCRIPTION: This function converts an YIQ image into RGB
 *		format. The parameter ot should be previously
 *		initialized to PPM format. Green value is stored in
 *		ot->jarr field, Blue in ot->karr field and 
 *		Red in ot->iarr field. 
 *
 *	RETURN VALUE: 0 if successful, non-zero upon failure
 ***********************************************************************/
int CIPL_yiqtorgb(THEADER *it, THEADER *ot)
{
     int i, j, max;
     double R, G, B, Y,I,Q;
     ot->img.TYPE = PPM;
     do {
	  max = HIPL_Getpart(it);
	  if (max < 0) {
	       HIPL_Error("Error in Getpart - incomplete read");
	       return(max);
	  }

	  for (i=0; i<max; i++) {
	       for (j=0; j<it->img.COLS; j++) {
		    Y = it->iarr[i][j];
		    I = it->jarr[i][j];
		    Q = it->karr[i][j];
		    R = 1 * Y + 0.956 * I + 0.621 * Q;
		    ot->iarr[i][j] = (int)(R+0.5);
		    G = 1 * Y  -0.272 * I  -0.647 * Q;
		    ot->jarr[i][j] = (int)(G+0.5);
		    B = 1 * Y  -1.105 * I + 1.702 * Q;
		    ot->karr[i][j] = (int)(B+0.5);

		    /* These checks are needed because of limited 
		       precision for Y, I, Q values 
		       can be as high as
		       261 on some occasions */
		    if (ot->iarr[i][j] > 255)
			 ot->iarr[i][j] = 255;
		    if (ot->jarr[i][j] > 255)
			 ot->jarr[i][j] = 255;
		    if (ot->karr[i][j] > 255)
			 ot->karr[i][j] = 255;

		    /* Just for debugging 
		    if ((i == 15) && (j == 15)) {
			 printf("Y, I, Q: %d, %d, %d\t\t", it->jarr[i][j],
				it->karr[i][j], it->iarr[i][j]);
			 printf("R, G, B: %d, %d, %d\n", ot->iarr[i][j],
				ot->jarr[i][j], ot->karr[i][j]);
		    }*/
	       }
	  }
	  
	  max = HIPL_Putpart(ot);
	  if (max < 0) {
	       HIPL_Error("Error in Putpart - incomplete write");
	       return(max);
	  }
	  
	  if (ot->P.nleft == 0)
	       max = -1;
     } while (max >= 0);

     HIPL_Refresh(it, 0);
     HIPL_Refresh(ot, 0);

     return(0);
}
